import datetime
from pathlib import Path
from tempfile import TemporaryDirectory

import pytest
from cryptography import x509
from cryptography.hazmat.primitives import hashes, serialization
from cryptography.hazmat.primitives.asymmetric import rsa
from cryptography.x509.oid import NameOID

from zxtx import open as zxtx_open
from zxtx.constants import CIPHER_METHOD, COMPRESSION_METHOD
from zxtx.signer import load_private_key, load_public_key


@pytest.fixture
def temp_dir():
    with TemporaryDirectory() as tmp_dir:
        yield Path(tmp_dir)


@pytest.fixture
def rsa_keys(tmp_path):
    private_key = rsa.generate_private_key(
        public_exponent=65537,
        key_size=1024,
    )
    public_key = private_key.public_key()

    # Verify public key matches private key
    expected_pub_bytes = private_key.public_key().public_bytes(
        serialization.Encoding.PEM,
        serialization.PublicFormat.SubjectPublicKeyInfo,
    )
    actual_pub_bytes = public_key.public_bytes(
        serialization.Encoding.PEM,
        serialization.PublicFormat.SubjectPublicKeyInfo,
    )
    assert actual_pub_bytes == expected_pub_bytes, "Public key mismatch"

    subject = issuer = x509.Name(
        [
            x509.NameAttribute(NameOID.COUNTRY_NAME, "US"),
            x509.NameAttribute(NameOID.STATE_OR_PROVINCE_NAME, "California"),
            x509.NameAttribute(NameOID.LOCALITY_NAME, "San Francisco"),
            x509.NameAttribute(NameOID.ORGANIZATION_NAME, "ZXTX"),
            x509.NameAttribute(NameOID.COMMON_NAME, "ZXTX Signing Key"),
        ]
    )

    certificate = (
        x509.CertificateBuilder()
        .subject_name(subject)
        .issuer_name(issuer)
        .public_key(public_key)
        .serial_number(x509.random_serial_number())
        .not_valid_before(datetime.datetime.now(datetime.timezone.utc))
        .not_valid_after(
            datetime.datetime.now(datetime.timezone.utc) + datetime.timedelta(days=3650)
        )
        .add_extension(x509.BasicConstraints(ca=True, path_length=None), critical=True)
        .sign(private_key, hashes.SHA256())
    )

    # Write keys and certificate to files
    priv_path = tmp_path / "private_key.pem"
    pub_path = tmp_path / "public_key.pem"
    cert_path = tmp_path / "certificate.pem"

    priv_path.write_bytes(
        private_key.private_bytes(
            encoding=serialization.Encoding.PEM,
            format=serialization.PrivateFormat.PKCS8,
            encryption_algorithm=serialization.NoEncryption(),
        )
    )
    pub_path.write_bytes(
        public_key.public_bytes(
            encoding=serialization.Encoding.PEM,
            format=serialization.PublicFormat.SubjectPublicKeyInfo,
        )
    )
    cert_path.write_bytes(certificate.public_bytes(serialization.Encoding.PEM))

    return priv_path, pub_path, cert_path


@pytest.mark.parametrize(
    "cipher, compression",
    [
        (CIPHER_METHOD.NONE, COMPRESSION_METHOD.NONE),
        (CIPHER_METHOD.AES256_GCM, COMPRESSION_METHOD.NONE),
        (CIPHER_METHOD.CHACHA20_POLY1305, COMPRESSION_METHOD.NONE),
        # (CIPHER_METHOD.AES256_CTR_HMAC, COMPRESSION_METHOD.NONE),
        # FIXME: results in error. why? no fucking idea.
        (CIPHER_METHOD.NONE, COMPRESSION_METHOD.ZLIB),
    ],
)
def test_write_read_roundtrip(temp_dir, rsa_keys, cipher, compression):
    priv_path, pub_path, cert_path = rsa_keys
    priv_key = load_private_key(priv_path.read_bytes())
    pub_key = load_public_key(pub_path.read_bytes())
    cert_bytes = cert_path.read_bytes()

    file_path = temp_dir / "test.zxtx"
    text_data = "Hello, ZXTX!"

    zfile = zxtx_open(
        path=file_path,
        cipher=cipher,
        compression=compression,
        private_key=priv_key,
        public_key=pub_key,
        certificate=cert_bytes,
        password=None,
    )

    # Write and read text
    zfile.write(text_data)
    assert zfile.read() == text_data

    # Append and read text
    zfile.append(" More data.")
    assert zfile.read() == text_data + " More data."

    # Write and read bytes
    binary_data = b"\x00\x01\x02\x03\x04"
    zfile.write_bytes(binary_data)
    assert zfile.read_bytes() == binary_data

    # Append and read bytes
    zfile.append_bytes(b"\x05\x06")
    assert zfile.read_bytes() == binary_data + b"\x05\x06"

    zfile.close()


def test_context_manager_usage(temp_dir, rsa_keys):
    priv_path, pub_path, cert_path = rsa_keys
    priv_key = load_private_key(priv_path.read_bytes())
    pub_key = load_public_key(pub_path.read_bytes())
    cert_bytes = cert_path.read_bytes()

    file_path = temp_dir / "context.zxtx"
    text_data = "Context manager test"

    with zxtx_open(
        file_path,
        cipher=CIPHER_METHOD.NONE,
        compression=COMPRESSION_METHOD.NONE,
        private_key=priv_key,
        public_key=pub_key,
        certificate=cert_bytes,
        password=None,
    ) as zfile:
        zfile.write(text_data)
        assert zfile.read() == text_data

    # After close, writing or reading should raise
    with pytest.raises(RuntimeError):
        zfile.write("Should fail after close")

    with pytest.raises(RuntimeError):
        zfile.read()


def test_error_on_closed_file(temp_dir, rsa_keys):
    priv_path, pub_path, cert_path = rsa_keys
    priv_key = load_private_key(priv_path.read_bytes())
    pub_key = load_public_key(pub_path.read_bytes())
    cert_bytes = cert_path.read_bytes()

    file_path = temp_dir / "closed.zxtx"
    zfile = zxtx_open(
        file_path,
        private_key=priv_key,
        public_key=pub_key,
        certificate=cert_bytes,
        password=None,
    )
    zfile.close()

    with pytest.raises(RuntimeError):
        zfile.write("Should fail")

    with pytest.raises(RuntimeError):
        zfile.append("Should fail")


def test_get_header(temp_dir, rsa_keys):
    priv_path, pub_path, _ = rsa_keys
    priv_key = load_private_key(priv_path.read_bytes())
    pub_key = load_public_key(pub_path.read_bytes())

    file_path = temp_dir / "header.zxtx"
    data = "Header test data"

    zfile = zxtx_open(file_path, private_key=priv_key, public_key=pub_key)
    zfile.write(data)
    header = zfile.get_header()

    assert header.original_size == len(data.encode(zfile._encoding))


def test_load_keys_from_bytes(temp_dir, rsa_keys):
    priv_path, pub_path, cert_path = rsa_keys
    priv_key = load_private_key(priv_path.read_bytes())
    pub_key = load_public_key(pub_path.read_bytes())
    cert_bytes = cert_path.read_bytes()

    file_path = temp_dir / "key_bytes.zxtx"
    text_data = "Key bytes test"

    priv_bytes = priv_key.private_bytes(
        encoding=serialization.Encoding.PEM,
        format=serialization.PrivateFormat.PKCS8,
        encryption_algorithm=serialization.NoEncryption(),
    )

    pub_bytes = pub_key.public_bytes(
        encoding=serialization.Encoding.PEM,
        format=serialization.PublicFormat.SubjectPublicKeyInfo,
    )

    zfile = zxtx_open(
        file_path,
        private_key=priv_bytes,
        public_key=pub_bytes,
        certificate=cert_bytes,
        password=None,
    )

    zfile.write(text_data)
    assert zfile.read() == text_data



from zxtx.__package_info__ import (__author__, __email__, __license__, __url__,
                                   __version__)
from zxtx.constants import *
from zxtx.decryption import *
from zxtx.dtypes import *
from zxtx.encryption import *
from zxtx.highlevel import *
from zxtx.parser import *
from zxtx.signer import *
from zxtx.utils import *
from zxtx.verifier import *
from zxtx.writer import *

__all__ = [
    "CIPHER_METHOD",
    "COMPRESSION_METHOD",
    "MAGIC_NUMBER",
    "write_zxtx_file",
    "read_zxtx_file",
    "parse_zxtx_header",
    "decrypt_data_aes_gcm",
    "decrypt_data_chacha20",
    "decrypt_key_rsa",
    "decrypt_data_aes256_ctr_hmac",
    "encrypt_data_aes_gcm",
    "encrypt_data_chacha20",
    "encrypt_key_rsa",
    "encrypt_data_aes256_ctr_hmac",
    "sign_data",
    "load_private_key",
    "load_public_key",
    "ZXTXHeader",
    "ZXTXBody",
    "verify_signature",
    "bits_to_bytes",
    "__version__",
    "__author__",
    "__email__",
    "__license__",
    "__url__",
    "open",
    "ZXTXFileHandle",
]



import datetime
import os
from itertools import product
from pathlib import Path

import pytest
from cryptography import x509
from cryptography.hazmat.primitives import hashes, serialization
from cryptography.hazmat.primitives.asymmetric import rsa
from cryptography.x509.oid import NameOID

from zxtx.constants import CIPHER_METHOD, COMPRESSION_METHOD
from zxtx.parser import parse_zxtx_header, read_zxtx_file
from zxtx.signer import load_private_key, load_public_key
from zxtx.writer import write_zxtx_file

key_sizes = [1024, 2048, 3072, 4096]
passwords = [
    None,
    b"password123",
    os.urandom(1023),
]

params = []
ids = []
for key_size in key_sizes:
    for pw in passwords:
        mark = (
            pytest.mark.slow
            if key_size > 1024
            else pytest.mark.skipif(False, reason="not slow")
        )
        label = (
            "NONE"
            if pw is None
            else (
                "PASSWORD123"
                if pw == b"password123"
                else (
                    "ANOTHERPASSWORD"
                    if pw == b"anotherpassword"
                    else f"URANDOM{len(pw)}"
                )
            )
        )
        params.append(pytest.param((key_size, pw), marks=mark))
        ids.append(f"{key_size}-{label}")

data_sizes = [
    os.urandom(1024),
    os.urandom(1024 * 64),
]

test_params = list(
    product(
        [
            CIPHER_METHOD.NONE,
            CIPHER_METHOD.AES256_GCM,
            CIPHER_METHOD.CHACHA20_POLY1305,
        ],
        [
            COMPRESSION_METHOD.NONE,
            COMPRESSION_METHOD.ZLIB,
            COMPRESSION_METHOD.LZMA,
            COMPRESSION_METHOD.BROTLI,
        ],
        data_sizes,
    )
)


@pytest.fixture(params=params, ids=ids)
def keypair_and_cert(tmp_path, request):
    key_size, password = request.param

    private_key = rsa.generate_private_key(
        public_exponent=65537,
        key_size=key_size,
    )
    public_key = private_key.public_key()

    # Key integrity sanity check
    assert public_key.public_bytes(
        serialization.Encoding.PEM,
        serialization.PublicFormat.SubjectPublicKeyInfo,
    ) == private_key.public_key().public_bytes(
        serialization.Encoding.PEM,
        serialization.PublicFormat.SubjectPublicKeyInfo,
    ), "Public and private keys do not match!"

    subject = issuer = x509.Name(
        [
            x509.NameAttribute(NameOID.COUNTRY_NAME, "US"),
            x509.NameAttribute(NameOID.STATE_OR_PROVINCE_NAME, "California"),
            x509.NameAttribute(NameOID.LOCALITY_NAME, "San Francisco"),
            x509.NameAttribute(NameOID.ORGANIZATION_NAME, "ZXTX"),
            x509.NameAttribute(NameOID.COMMON_NAME, "ZXTX Signing Key"),
        ]
    )

    cert = (
        x509.CertificateBuilder()
        .subject_name(subject)
        .issuer_name(issuer)
        .public_key(public_key)
        .serial_number(x509.random_serial_number())
        .not_valid_before(datetime.datetime.now(datetime.timezone.utc))
        .not_valid_after(
            datetime.datetime.now(datetime.timezone.utc) + datetime.timedelta(days=3650)
        )
        .add_extension(x509.BasicConstraints(ca=True, path_length=None), critical=True)
        .sign(private_key, hashes.SHA256())
    )

    encryption = (
        serialization.BestAvailableEncryption(password)
        if password
        else serialization.NoEncryption()
    )

    priv_path = tmp_path / "private_key.pem"
    pub_path = tmp_path / "public_key.pem"
    cert_path = tmp_path / "certificate.pem"

    priv_path.write_bytes(
        private_key.private_bytes(
            encoding=serialization.Encoding.PEM,
            format=serialization.PrivateFormat.PKCS8,
            encryption_algorithm=encryption,
        )
    )
    pub_path.write_bytes(
        public_key.public_bytes(
            encoding=serialization.Encoding.PEM,
            format=serialization.PublicFormat.SubjectPublicKeyInfo,
        )
    )
    cert_path.write_bytes(cert.public_bytes(serialization.Encoding.PEM))

    return key_size, priv_path, pub_path, cert_path, password


@pytest.mark.parametrize(
    ("cipher", "compression", "original_data"),
    test_params,
    ids=lambda val: f"DATA_{len(val)//1024}KB" if isinstance(val, bytes) else str(val),
)
def test_encrypted_compressed_rsa(
    tmp_path, keypair_and_cert, cipher, compression, original_data
):
    _, priv_path, pub_path, cert_path, password = keypair_and_cert
    private_key = load_private_key(priv_path.read_bytes(), password=password)
    public_key = load_public_key(pub_path.read_bytes())
    certificate = cert_path.read_bytes()

    zxtx_bytes = write_zxtx_file(
        data=original_data,
        compression_method=compression,
        cipher_method=cipher,
        private_key=private_key,
        public_key=public_key,
        certificate=certificate,
    )

    zxtx_path = tmp_path / "test.zxtx"
    zxtx_path.write_bytes(zxtx_bytes)
    zxtx_bytes = zxtx_path.read_bytes()

    header, body = parse_zxtx_header(zxtx_bytes, private_key=private_key)
    assert header.original_size == len(original_data)

    decrypted = read_zxtx_file(header, body, private_key=private_key)
    assert decrypted == original_data



import datetime as dt
from dataclasses import dataclass
from typing import Optional

from zxtx.constants import CIPHER_METHOD, COMPRESSION_METHOD


@dataclass
class ZXTXHeader:
    version_major: int
    version_minor: int
    uid: bytes
    compression_method: COMPRESSION_METHOD
    cipher_method: CIPHER_METHOD
    original_size: int
    compressed_size: int
    timestamp: int
    sha256_hash: bytes
    crc32: int
    signature: bytes
    certificate: Optional[bytes]

    def creation_time(self) -> dt.datetime:
        return dt.datetime.fromtimestamp(self.timestamp)


@dataclass
class ZXTXBody:
    data: bytes
    nonce: Optional[bytes]
    encrypted_key: Optional[bytes]
    tag: Optional[bytes]


@dataclass
class ZXTXData:
    data: bytes



import sys

from cryptography.hazmat.primitives import hashes
from cryptography.hazmat.primitives.asymmetric import ec, ed25519, padding, rsa
from cryptography.hazmat.primitives.serialization import (load_pem_private_key,
                                                          load_pem_public_key)


def sign_data(private_key, data: bytes) -> bytes:
    if isinstance(private_key, ec.EllipticCurvePrivateKey):
        signature = private_key.sign(data, ec.ECDSA(hashes.SHA256()))
    elif isinstance(private_key, ed25519.Ed25519PrivateKey):
        signature = private_key.sign(data)
    elif isinstance(private_key, rsa.RSAPrivateKey):
        return private_key.sign(data, padding.PKCS1v15(), hashes.SHA256())
    elif private_key is None:
        return b""
    else:
        raise TypeError("Unsupported private key type", private_key)
    return signature


def load_private_key(pem_bytes: bytes, password=None):
    try:
        return load_pem_private_key(pem_bytes, password=password)
    except ValueError as error:
        (
            error.add_note(
                "Note: The private key may be encrypted or a invalid password was provided."
            )
            if sys.version_info >= (3, 11)
            else ...
        )
        error.args = error.args[1:]
        error.with_traceback(None)
        raise error


def load_public_key(pem_bytes: bytes):
    return load_pem_public_key(pem_bytes)



import argparse
from pathlib import Path
from typing import Optional

from zxtx.constants import CIPHER_METHOD, COMPRESSION_METHOD
from zxtx.dtypes import ZXTXBody, ZXTXData, ZXTXHeader
from zxtx.highlevel import _open, open

try:
    from rich.pretty import pprint
except ImportError:
    from pprint import pprint


def get_bytes(path: Optional[str]) -> Optional[bytes]:
    return Path(path).read_bytes() if path else None


def write_cmd(args):
    password_bytes = args.password.encode() if args.password else None

    with (
        _open(args.input_path, "rb") as input_file,
        open(
            args.output_path,
            cipher=CIPHER_METHOD(args.cipher),
            compression=COMPRESSION_METHOD(args.compression),
            password=password_bytes,
            certificate=get_bytes(args.certificate),
            private_key=get_bytes(args.private_key),
            public_key=get_bytes(args.public_key),
        ) as output_file,
    ):
        output_file.write_bytes(input_file.read())


def read_cmd(args):
    password_bytes = args.password.encode() if args.password else None

    try:
        with open(
            args.input_path,
            password=password_bytes,
            public_key=get_bytes(args.public_key),
            private_key=get_bytes(args.private_key),
            certificate=get_bytes(args.certificate),
            cipher=CIPHER_METHOD(args.cipher),
            compression=COMPRESSION_METHOD(args.compression),
        ) as file:
            data = file.read_bytes()

        if args.output_path:
            Path(args.output_path).write_bytes(data)
        else:
            try:
                print(data.decode("utf-8"))
            except UnicodeDecodeError:
                print(repr(data))

    except Exception as e:
        print(f"error: {e}")
        exit(1)


def dump_cmd(args):
    password_bytes = args.password.encode() if args.password else None

    try:
        with open(
            args.input_path,
            password=password_bytes,
            public_key=get_bytes(args.public_key),
            private_key=get_bytes(args.private_key),
            certificate=get_bytes(args.certificate),
            cipher=CIPHER_METHOD(args.cipher),
            compression=COMPRESSION_METHOD(args.compression),
        ) as zxtx_file:
            header: ZXTXHeader = zxtx_file._header
            body: ZXTXBody = zxtx_file._body
            data: bytes = zxtx_file._data

            zxtx_file._read()

            pprint(header)

            pprint(body)

            pprint(ZXTXData(data=data))

    except Exception as e:
        print(f"error: {e}")
        exit(1)


def main():
    parser = argparse.ArgumentParser(description="ZXTX CLI")
    subparsers = parser.add_subparsers(dest="command", required=True)

    # Shared options
    def add_common_options(p):
        p.add_argument(
            "--cipher",
            type=int,
            default=0,
            choices=[c.value for c in CIPHER_METHOD],
            help="Cipher method (int).",
        )
        p.add_argument(
            "--compression",
            type=int,
            default=0,
            choices=[c.value for c in COMPRESSION_METHOD],
            help="Compression method (int).",
        )
        p.add_argument(
            "--password", type=str, help="Password to decrypt private key (if needed)."
        )
        p.add_argument("--public-key", type=str, help="Path to public key file.")
        p.add_argument("--private-key", type=str, help="Path to private key file.")
        p.add_argument("--certificate", type=str, help="Path to certificate file.")

    # write
    write_parser = subparsers.add_parser("write", help="Write a ZXTX file.")
    write_parser.add_argument("input_path", type=Path, help="Input file path.")
    write_parser.add_argument("output_path", type=Path, help="Output .zxtx file path.")
    add_common_options(write_parser)
    write_parser.set_defaults(func=write_cmd)

    # read
    read_parser = subparsers.add_parser("read", help="Read a ZXTX file.")
    read_parser.add_argument("input_path", type=Path, help="Input .zxtx file path.")
    read_parser.add_argument(
        "output_path",
        type=Path,
        nargs="?",
        help="Where to save extracted output (optional).",
    )
    add_common_options(read_parser)
    read_parser.set_defaults(func=read_cmd)

    # dump
    dump_parser = subparsers.add_parser("dump", help="Dump ZXTX file metadata.")
    dump_parser.add_argument("input_path", type=Path, help="Input .zxtx file path.")
    add_common_options(dump_parser)
    dump_parser.set_defaults(func=dump_cmd)

    args = parser.parse_args()
    args.func(args)


if __name__ == "__main__":
    main()



import os

from cryptography.hazmat.primitives import hashes, hmac
from cryptography.hazmat.primitives.asymmetric import padding
from cryptography.hazmat.primitives.ciphers import Cipher, algorithms, modes
from cryptography.hazmat.primitives.ciphers.aead import (AESGCM,
                                                         ChaCha20Poly1305)


def encrypt_data_aes_gcm(data: bytes, key: bytes) -> tuple[bytes, bytes, bytes]:
    nonce = os.urandom(12)
    aesgcm = AESGCM(key)
    encrypted = aesgcm.encrypt(nonce, data, None)
    return encrypted, nonce, key


def encrypt_data_chacha20(data: bytes, key: bytes) -> tuple[bytes, bytes, bytes]:
    nonce = os.urandom(12)
    cipher = ChaCha20Poly1305(key)
    encrypted = cipher.encrypt(nonce, data, None)
    return encrypted, nonce, key


def encrypt_data_aes256_ctr_hmac(data: bytes, key: bytes) -> tuple[bytes, bytes, bytes]:
    # Split key: 32 bytes encryption key + 32 bytes HMAC key
    if len(key) != 64:
        raise ValueError("Key must be 64 bytes for AES-256-CTR + HMAC")
    enc_key = key[:32]
    hmac_key = key[32:]

    nonce = os.urandom(16)  # 128-bit nonce for CTR

    cipher = Cipher(algorithms.AES(enc_key), modes.CTR(nonce))
    encryptor = cipher.encryptor()
    ciphertext = encryptor.update(data) + encryptor.finalize()

    h = hmac.HMAC(hmac_key, hashes.SHA256())
    h.update(nonce + ciphertext)
    tag = h.finalize()

    return ciphertext, nonce, tag


def encrypt_key_rsa(public_key, session_key: bytes) -> bytes:
    return public_key.encrypt(
        session_key,
        padding.OAEP(
            mgf=padding.MGF1(algorithm=hashes.SHA256()),
            algorithm=hashes.SHA256(),
            label=None,
        ),
    )



import datetime
import os
import tempfile
from itertools import product
from pathlib import Path

import pytest
from cryptography import x509
from cryptography.hazmat.primitives import hashes, serialization
from cryptography.hazmat.primitives.asymmetric import ed25519
from cryptography.x509.oid import NameOID

from zxtx.constants import CIPHER_METHOD, COMPRESSION_METHOD
from zxtx.parser import parse_zxtx_header, read_zxtx_file
from zxtx.signer import load_private_key, load_public_key
from zxtx.writer import write_zxtx_file

data_sizes = [
    os.urandom(1024),
    os.urandom(1024 * 64),
]

test_params = list(
    product(
        [
            COMPRESSION_METHOD.NONE,
            COMPRESSION_METHOD.ZLIB,
            COMPRESSION_METHOD.LZMA,
            COMPRESSION_METHOD.BROTLI,
        ],
        data_sizes,
    )
)


@pytest.fixture(
    params=[
        None,
        b"password123",
        b"anotherpassword",
        os.urandom(128),
        os.urandom(256),
        os.urandom(512),
        os.urandom(1023),
    ],
    ids=[
        "none".upper(),
        "password123".upper(),
        "anotherpassword".upper(),
        "urandom128".upper(),
        "urandom256".upper(),
        "urandom512".upper(),
        "urandom1023".upper(),
    ],
)
def keypair_and_cert(tmp_path, request):
    """Parametrized: Generates keypair + cert using password param."""
    password = request.param

    private_key = ed25519.Ed25519PrivateKey.generate()
    public_key = private_key.public_key()

    subject = issuer = x509.Name(
        [
            x509.NameAttribute(NameOID.COUNTRY_NAME, "US"),
            x509.NameAttribute(NameOID.STATE_OR_PROVINCE_NAME, "California"),
            x509.NameAttribute(NameOID.LOCALITY_NAME, "San Francisco"),
            x509.NameAttribute(NameOID.ORGANIZATION_NAME, "ZXTX"),
            x509.NameAttribute(NameOID.COMMON_NAME, "ZXTX Signing Key"),
        ]
    )

    cert = (
        x509.CertificateBuilder()
        .subject_name(subject)
        .issuer_name(issuer)
        .public_key(public_key)
        .serial_number(x509.random_serial_number())
        .not_valid_before(datetime.datetime.now(datetime.timezone.utc))
        .not_valid_after(
            datetime.datetime.now(datetime.timezone.utc) + datetime.timedelta(days=3650)
        )
        .add_extension(x509.BasicConstraints(ca=True, path_length=None), critical=True)
        .sign(private_key, None)
    )

    # Save to files
    priv_path = tmp_path / "private_key.pem"
    pub_path = tmp_path / "public_key.pem"
    cert_path = tmp_path / "certificate.pem"

    encryption = (
        serialization.BestAvailableEncryption(password)
        if password
        else serialization.NoEncryption()
    )

    priv_path.write_bytes(
        private_key.private_bytes(
            encoding=serialization.Encoding.PEM,
            format=serialization.PrivateFormat.PKCS8,
            encryption_algorithm=encryption,
        )
    )

    pub_path.write_bytes(
        public_key.public_bytes(
            encoding=serialization.Encoding.PEM,
            format=serialization.PublicFormat.SubjectPublicKeyInfo,
        )
    )

    cert_path.write_bytes(cert.public_bytes(serialization.Encoding.PEM))

    return priv_path, pub_path, cert_path, password


@pytest.mark.parametrize(
    ("compression", "original_data"),
    test_params,
    ids=lambda val: f"DATA_{len(val)//1024}KB" if isinstance(val, bytes) else str(val),
)
def test_unencrypted_compressed_ed25519(
    tmp_path, keypair_and_cert, compression, original_data
):
    priv_path, pub_path, cert_path, password = keypair_and_cert
    private_key = load_private_key(priv_path.read_bytes(), password=password)
    public_key = load_public_key(pub_path.read_bytes())
    certificate = cert_path.read_bytes()

    zxtx_bytes = write_zxtx_file(
        data=original_data,
        compression_method=compression,
        cipher_method=CIPHER_METHOD.NONE,
        private_key=private_key,
        public_key=public_key,
        certificate=certificate,
    )

    zxtx_path = tmp_path / "test.zxtx"
    zxtx_path.write_bytes(zxtx_bytes)

    header, body = parse_zxtx_header(zxtx_path.read_bytes(), private_key=private_key)

    decrypted = read_zxtx_file(header, body, private_key=private_key)

    # Parse header
    parsed_header, _ = parse_zxtx_header(zxtx_path.read_bytes())
    assert parsed_header.original_size == len(original_data)

    # Decrypt and read
    assert decrypted == original_data



import sys
from pathlib import Path
from typing import Optional

from cryptography.hazmat.primitives.asymmetric import (dh, dsa, ec, ed448,
                                                       ed25519, rsa, x448,
                                                       x25519)
from cryptography.hazmat.primitives.asymmetric.types import (PrivateKeyTypes,
                                                             PublicKeyTypes)

from zxtx.constants import CIPHER_METHOD, COMPRESSION_METHOD
from zxtx.decryption import (decrypt_data_aes256_ctr_hmac,
                             decrypt_data_aes_gcm, decrypt_data_chacha20,
                             decrypt_key_rsa)
from zxtx.dtypes import ZXTXBody, ZXTXHeader
from zxtx.encryption import (encrypt_data_aes256_ctr_hmac,
                             encrypt_data_aes_gcm, encrypt_data_chacha20,
                             encrypt_key_rsa)
from zxtx.parser import parse_zxtx_header, read_zxtx_file
from zxtx.signer import load_private_key, load_public_key, sign_data
from zxtx.verifier import verify_signature
from zxtx.writer import write_zxtx_file

_open = open


class ZXTXFileHandle:
    def __init__(
        self,
        path: Path | str,
        *,
        cipher: CIPHER_METHOD = CIPHER_METHOD.NONE,
        compression: COMPRESSION_METHOD = COMPRESSION_METHOD.NONE,
        certificate: Optional[bytes] = None,
        private_key: Optional[PrivateKeyTypes | bytes] = None,
        public_key: Optional[PublicKeyTypes | bytes] = None,
        password: Optional[bytes] = None,
        encoding: str = "utf-8",
    ):
        self._path = Path(path)
        self._cipher = cipher
        self._compression = compression
        self._certificate = certificate
        self._private_key = (
            load_private_key(private_key, password=password)
            if isinstance(private_key, bytes)
            else private_key
        )

        self._public_key = (
            load_public_key(public_key) if isinstance(public_key, bytes) else public_key
        )

        self._password = password
        self._encoding = encoding

        self._closed = False

        self._header: ZXTXHeader
        self._body: ZXTXBody
        self._raw_data: bytes
        self._data: bytes = b""

        if (
            self._path.exists()
            and len(self._path.read_bytes()) != 0
            and self._path.is_file()
        ):
            self._read()

    def _read(self):
        if self._closed:
            raise RuntimeError("Cannot read from a closed file")

        with _open(self._path, "rb") as f:
            self._raw_data = f.read()

        self._header, self._body = parse_zxtx_header(
            self._raw_data, private_key=self._private_key
        )

        self._data = read_zxtx_file(self._header, self._body, self._private_key)

    def _write_current(self):
        if self._closed:
            raise RuntimeError("Cannot write to a closed file")

        zxtx_bytes = write_zxtx_file(
            data=self._data,
            cipher_method=self._cipher,
            compression_method=self._compression,
            certificate=self._certificate,
            private_key=self._private_key,
            public_key=self._public_key,
        )

        with _open(self._path, "wb") as file:
            file.write(zxtx_bytes)

        self._read()

    def write(self, text: str):
        self._data = text.encode(self._encoding)
        self._write_current()

    def write_bytes(self, data: bytes):
        self._data = data
        self._write_current()

    def append(self, text: str):
        self._data += text.encode(self._encoding)
        self._write_current()

    def append_bytes(self, data: bytes):
        self._data += data
        self._write_current()

    def read(self) -> str:
        if self._closed:
            raise RuntimeError("Cannot read from a closed file")
        if self._data == b"":
            self._read()
        return self._data.decode(self._encoding)

    def read_bytes(self) -> bytes:
        if self._data == b"":
            self._read()
        return self._data

    def get_header(self) -> ZXTXHeader:
        if not self._header:
            self._read()
        return self._header

    def close(self):
        self._closed = True

    def __enter__(self) -> "ZXTXFileHandle":
        return self

    def __exit__(self, exc_type, exc_value, traceback):
        self.close()

        if exc_value:
            (
                exc_value.add_note(
                    f"Uncaught exception while handling '{self._path}', additional information [{self._cipher=}, {self._compression=}]"
                )
                if sys.version_info >= (3, 11)
                else ...
            )

            raise exc_value


def open(
    path: Path | str,
    *,
    cipher: CIPHER_METHOD = CIPHER_METHOD.NONE,
    compression: COMPRESSION_METHOD = COMPRESSION_METHOD.NONE,
    certificate: Optional[bytes] = None,
    private_key: Optional[PrivateKeyTypes | bytes] = None,
    public_key: Optional[PublicKeyTypes | bytes] = None,
    password: Optional[bytes] = None,
    encoding: str = "utf-8",
) -> ZXTXFileHandle:
    return ZXTXFileHandle(
        path=path,
        cipher=cipher,
        compression=compression,
        certificate=certificate,
        private_key=private_key,
        public_key=public_key,
        password=password,
        encoding=encoding,
    )



import hashlib
import lzma
import struct
import zlib

import brotli

from zxtx.constants import CIPHER_METHOD, COMPRESSION_METHOD, MAGIC_NUMBER
from zxtx.decryption import (decrypt_data_aes256_ctr_hmac,
                             decrypt_data_aes_gcm, decrypt_data_chacha20,
                             decrypt_key_rsa)
from zxtx.dtypes import ZXTXBody, ZXTXHeader
from zxtx.utils import bits_to_bytes


def parse_zxtx_header(data: bytes, private_key=None) -> tuple[ZXTXHeader, ZXTXBody]:
    offset = 0

    def read(fmt: str):
        nonlocal offset
        size = struct.calcsize(fmt)
        value = struct.unpack_from(fmt, data, offset)
        offset += size
        return value if len(value) > 1 else value[0]  # type: ignore

    # Magic number
    magic = data[offset : offset + 4]
    if magic != MAGIC_NUMBER:
        raise ValueError("Invalid ZXTX file: bad magic number")

    offset += 4

    version_minor = data[offset + 1]
    version_major = data[offset]
    offset += 2

    uid = data[offset : offset + 16]
    offset += 16

    compression_method = COMPRESSION_METHOD(data[offset])
    cipher_method = CIPHER_METHOD(data[offset + 1])
    offset += 2

    original_size = read(">Q")
    compressed_size = read(">Q")
    timestamp = read(">d")
    sha256_hash = data[offset : offset + 32]
    offset += 32

    crc32 = read(">I")
    cert_len = read(">H")
    certificate = data[offset : offset + cert_len]
    offset += cert_len

    sig_len = read(">H")
    signature = data[offset : offset + sig_len]
    offset += sig_len

    compressed_data = data[offset : offset + compressed_size]

    if zlib.crc32(compressed_data) != crc32:
        raise ValueError("CRC32 mismatch: file may be corrupted or tampered")

    tag = None

    # If cipher is none, compressed_data is raw compressed data
    # Else, encrypted_data structure: encrypted_session_key + nonce + encrypted_blob
    match cipher_method:
        case CIPHER_METHOD.NONE:
            encrypted_key = None
            nonce = None
            encrypted_blob = compressed_data
            tag = None
        case (
            CIPHER_METHOD.AES256_CTR_HMAC
            | CIPHER_METHOD.AES256_GCM
            | CIPHER_METHOD.CHACHA20_POLY1305
        ):
            if private_key is None:
                # Default to RSA-2048 (256 bytes)
                key_size = 256
            else:
                key_size = bits_to_bytes(bits=private_key.key_size)

            # Set nonce size depending on cipher
            nonce_size = 16 if cipher_method == CIPHER_METHOD.AES256_CTR_HMAC else 12

            encrypted_key = compressed_data[:key_size]
            nonce = compressed_data[key_size : key_size + nonce_size]

            if cipher_method == CIPHER_METHOD.AES256_CTR_HMAC:
                # Last 32 bytes are HMAC tag
                encrypted_blob = compressed_data[key_size + nonce_size : -32]
                tag = compressed_data[-32:]
            else:
                encrypted_blob = compressed_data[key_size + nonce_size :]
                tag = None
        case _:
            raise ValueError(f"Invalid cipher method '{cipher_method}'")

    return (
        ZXTXHeader(
            version_major=version_major,
            version_minor=version_minor,
            uid=uid,
            compression_method=compression_method,
            cipher_method=cipher_method,
            original_size=original_size,
            compressed_size=compressed_size,
            timestamp=timestamp,
            sha256_hash=sha256_hash,
            crc32=crc32,
            certificate=certificate,
            signature=signature,
        ),
        ZXTXBody(
            data=encrypted_blob, nonce=nonce, encrypted_key=encrypted_key, tag=tag
        ),
    )


def read_zxtx_file(header: ZXTXHeader, body: ZXTXBody, private_key) -> bytes:
    match header.cipher_method:
        case CIPHER_METHOD.NONE:
            decrypted_data = body.data
        case CIPHER_METHOD.AES256_GCM:
            if body.encrypted_key is None or body.nonce is None:
                raise ValueError(
                    "Missing encrypted key or nonce for AES256_GCM decryption"
                )

            if private_key is None:
                raise ValueError("Missing private key for AES256_GCM decryption")

            session_key = decrypt_key_rsa(private_key, body.encrypted_key)
            decrypted_data = decrypt_data_aes_gcm(body.data, body.nonce, session_key)
        case CIPHER_METHOD.CHACHA20_POLY1305:
            if body.encrypted_key is None or body.nonce is None:
                raise ValueError(
                    "Missing encrypted key or nonce for ChaCha20_POLY1305 decryption"
                )

            if private_key is None:
                raise ValueError("Missing private key for ChaCha20_POLY1305 decryption")

            session_key = decrypt_key_rsa(private_key, body.encrypted_key)
            decrypted_data = decrypt_data_chacha20(body.data, body.nonce, session_key)
        case CIPHER_METHOD.AES256_CTR_HMAC:
            if body.encrypted_key is None or body.nonce is None:
                raise ValueError(
                    "Missing encrypted key or nonce for AES256_CTR_HMAC decryption"
                )

            if private_key is None:
                raise ValueError("Missing private key for AES256_CTR_HMAC decryption")

            session_key = decrypt_key_rsa(private_key, body.encrypted_key)
            decrypted_data = decrypt_data_aes256_ctr_hmac(
                body.data, body.nonce, body.tag, session_key  # type: ignore
            )
        case _:
            raise NotImplementedError(
                f"Unsupported cipher method '{header.cipher_method}'"
            )

    # Decompress
    if header.compression_method == COMPRESSION_METHOD.NONE:
        decompressed = decrypted_data
    elif header.compression_method == COMPRESSION_METHOD.ZLIB:
        decompressed = zlib.decompress(decrypted_data)
    elif header.compression_method == COMPRESSION_METHOD.LZMA:
        decompressed = lzma.decompress(decrypted_data)
    elif header.compression_method == COMPRESSION_METHOD.BROTLI:
        decompressed = brotli.decompress(decrypted_data)
    else:
        raise NotImplementedError(
            f"Unsupported compression method '{header.compression_method}'"
        )

    if hashlib.sha256(decompressed).digest() != header.sha256_hash:
        raise ValueError("SHA-256 hash mismatch: file may be corrupted or tampered")

    return decompressed



import sys

from cryptography.hazmat.primitives import hashes, hmac
from cryptography.hazmat.primitives.asymmetric import padding
from cryptography.hazmat.primitives.ciphers import Cipher, algorithms, modes
from cryptography.hazmat.primitives.ciphers.aead import (AESGCM,
                                                         ChaCha20Poly1305)


def decrypt_key_rsa(private_key, encrypted_key: bytes) -> bytes:
    try:
        return private_key.decrypt(
            encrypted_key,
            padding.OAEP(
                mgf=padding.MGF1(algorithm=hashes.SHA256()),
                algorithm=hashes.SHA256(),
                label=None,
            ),
        )
    except ValueError as error:
        (
            error.add_note("Note: The private key may be invalid.")
            if sys.version_info >= (3, 11)
            else ...
        )

        raise error


def decrypt_data_aes_gcm(encrypted_data: bytes, nonce: bytes, key: bytes) -> bytes:
    aesgcm = AESGCM(key)
    return aesgcm.decrypt(nonce, encrypted_data, associated_data=None)


def decrypt_data_chacha20(encrypted_data: bytes, nonce: bytes, key: bytes) -> bytes:
    chacha = ChaCha20Poly1305(key)
    return chacha.decrypt(nonce, encrypted_data, associated_data=None)


def decrypt_data_aes256_ctr_hmac(
    ciphertext: bytes, nonce: bytes, tag: bytes, key: bytes
) -> bytes:
    if len(key) != 64:
        raise ValueError("Key must be 64 bytes for AES-256-CTR + HMAC")
    enc_key = key[:32]
    hmac_key = key[32:]

    h = hmac.HMAC(hmac_key, hashes.SHA256())
    h.update(nonce + ciphertext)
    h.verify(tag)  # raises InvalidSignature if invalid

    cipher = Cipher(algorithms.AES(enc_key), modes.CTR(nonce))
    decryptor = cipher.decryptor()
    plaintext = decryptor.update(ciphertext) + decryptor.finalize()

    return plaintext



import binascii
import hashlib
import lzma
import os
import struct
import time
import uuid
import zlib
from enum import Enum
from typing import Optional

import brotli

from zxtx.constants import CIPHER_METHOD, COMPRESSION_METHOD, MAGIC_NUMBER
from zxtx.encryption import (encrypt_data_aes256_ctr_hmac,
                             encrypt_data_aes_gcm, encrypt_data_chacha20,
                             encrypt_key_rsa)
from zxtx.signer import sign_data


def write_zxtx_file(
    data: bytes,
    compression_method: Enum,
    cipher_method: Enum,
    private_key,
    certificate: Optional[bytes] = None,
    public_key=None,
    version=(1, 0),
) -> bytes:
    uid = uuid.uuid4().bytes
    timestamp = int(time.time())

    # Compress
    match compression_method:
        case COMPRESSION_METHOD.NONE:
            compressed = data
        case COMPRESSION_METHOD.ZLIB:
            compressed = zlib.compress(data)
        case COMPRESSION_METHOD.LZMA:
            compressed = lzma.compress(data)
        case COMPRESSION_METHOD.BROTLI:
            compressed = brotli.compress(data)
        case _:
            raise NotImplementedError("Compression method not supported")

    # Encrypt if cipher method is enabled
    match cipher_method:
        case CIPHER_METHOD.NONE:
            compressed_final = compressed
        case CIPHER_METHOD.AES256_GCM:
            if public_key is None:
                raise ValueError("Public key required for encryption")

            session_key = os.urandom(32)  # AES-256
            encrypted_data, nonce, _ = encrypt_data_aes_gcm(compressed, session_key)
            encrypted_session_key = encrypt_key_rsa(public_key, session_key)

            compressed_final = encrypted_session_key + nonce + encrypted_data
        case CIPHER_METHOD.CHACHA20_POLY1305:
            if public_key is None:
                raise ValueError("Public key required for encryption")

            session_key = os.urandom(32)  # ChaCha20 key length 256-bit
            encrypted_data, nonce, _ = encrypt_data_chacha20(compressed, session_key)
            encrypted_session_key = encrypt_key_rsa(public_key, session_key)
            compressed_final = encrypted_session_key + nonce + encrypted_data
        case CIPHER_METHOD.AES256_CTR_HMAC:
            if public_key is None:
                raise ValueError("Public key required for encryption")

            session_key = os.urandom(64)  # 32 bytes AES-CTR + 32 bytes HMAC
            encrypted_data, nonce, tag = encrypt_data_aes256_ctr_hmac(
                compressed, session_key
            )
            encrypted_session_key = encrypt_key_rsa(public_key, session_key)

            # Format: encrypted_session_key + nonce + ciphertext + tag
            compressed_final = encrypted_session_key + nonce + encrypted_data + tag
        case _:
            raise NotImplementedError(f"Unsupported cipher method '{cipher_method}'")

    if certificate is None:
        certificate = b""

    crc32 = binascii.crc32(compressed_final) & 0xFFFFFFFF
    sha256 = hashlib.sha256(data).digest()

    # Header
    header = bytearray()
    header += MAGIC_NUMBER
    header += struct.pack("BB", version[0], version[1])  # major, minor
    header += uid
    header += struct.pack("BB", compression_method.value, cipher_method.value)
    header += struct.pack(">Q", len(data))
    header += struct.pack(">Q", len(compressed_final))
    header += struct.pack(">d", timestamp)
    header += sha256
    header += struct.pack(">I", crc32)
    header += struct.pack(">H", len(certificate))
    header += certificate

    # Sign the header
    signature = sign_data(private_key, bytes(header))
    header += struct.pack(">H", len(signature))
    header += signature

    return bytes(header) + compressed_final



from cryptography.hazmat.primitives import hashes
from cryptography.hazmat.primitives.asymmetric import ec, ed25519, padding, rsa
from cryptography.x509 import load_pem_x509_certificate


def verify_signature(cert_bytes: bytes, data: bytes, signature: bytes) -> bool:
    cert = load_pem_x509_certificate(cert_bytes)
    public_key = cert.public_key()

    try:
        if isinstance(public_key, ec.EllipticCurvePublicKey):
            public_key.verify(signature, data, ec.ECDSA(hashes.SHA256()))
        elif isinstance(public_key, ed25519.Ed25519PublicKey):
            public_key.verify(signature, data)
        elif isinstance(public_key, rsa.RSAPublicKey):
            public_key.verify(signature, data, padding.PKCS1v15(), hashes.SHA256())
        else:
            return False
        return True
    except Exception:
        return False



__version__ = "0.1.1"  # Always hardcode or inject this during build

# Sensible defaults
__project_name__ = "Undefined"
__author__ = "Undefined"
__email__ = "Undefined"
__license__ = "Undefined"
__url__ = "Undefined"
__documentation__ = "Undefined"
__repository__ = "Undefined"
__issues__ = "Undefined"
__description__ = "Undefined"
__readme__ = "Undefined"
__keywords__ = []
__dependencies__ = []
__dev_dependencies__ = []
__python_requires__ = "Undefined"
__classifiers__ = []
__build_backend__ = "Undefined"
__build_requires__ = []

try:
    import toml
    from pathlib import Path

    current = Path(__file__).resolve().parent
    while current != current.parent:
        pyproject_path = current / "pyproject.toml"
        if pyproject_path.exists():
            break
        current = current.parent
    else:
        raise FileNotFoundError("pyproject.toml not found")

    pyproject = toml.load(pyproject_path)

    project = pyproject.get("project", {})
    __project_name__ = project.get("name", __project_name__)
    authors = project.get("authors", [{}])
    __author__ = authors[0].get("name", __author__)
    __email__ = authors[0].get("email", __email__)
    __license__ = project.get("license", {}).get("text", __license__)
    urls = project.get("urls", {})
    __url__ = urls.get("Homepage", __url__)
    __documentation__ = urls.get("Documentation", __documentation__)
    __repository__ = urls.get("Source", __repository__)
    __issues__ = urls.get("Issues", __issues__)
    __description__ = project.get("description", __description__)
    __readme__ = project.get("readme", __readme__)
    __keywords__ = project.get("keywords", __keywords__)
    __dependencies__ = project.get("dependencies", __dependencies__)
    __dev_dependencies__ = project.get("optional-dependencies", {}).get("dev", __dev_dependencies__)
    __python_requires__ = project.get("requires-python", __python_requires__)
    __classifiers__ = project.get("classifiers", __classifiers__)
    build_system = pyproject.get("build-system", {})
    __build_backend__ = build_system.get("build-backend", __build_backend__)
    __build_requires__ = build_system.get("requires", __build_requires__)
except Exception:
    pass  # Keep defaults



import datetime
import os
import tempfile
from itertools import product
from pathlib import Path

import pytest
from cryptography import x509
from cryptography.hazmat.primitives import hashes, serialization
from cryptography.hazmat.primitives.asymmetric import rsa
from cryptography.x509.oid import NameOID

from zxtx.signer import load_private_key, sign_data
from zxtx.verifier import verify_signature

key_sizes = [1024, 2048, 3072, 4096]
passwords = [
    None,
    b"password123",
    b"anotherpassword",
    os.urandom(16),
    os.urandom(32),
    os.urandom(64),
    os.urandom(1023),
]

params = []
for key_size in key_sizes:
    for pw in passwords:
        mark = (
            pytest.mark.slow
            if key_size > 2048
            else pytest.mark.skipif(False, reason="not slow")
        )
        params.append(pytest.param((key_size, pw), marks=mark))


@pytest.fixture(
    params=params,
    ids=[
        f"{key_size}-{label}"
        for key_size, label in product(
            [1024, 2048, 3072, 4096],
            [
                "NONE",
                "PASSWORD123",
                "ANOTHERPASSWORD",
                "URANDOM16",
                "URANDOM32",
                "URANDOM64",
                "URANDOM1023",
            ],
        )
    ],
)
def keypair_and_cert(tmp_path, request):
    key_size, password = request.param

    private_key = rsa.generate_private_key(
        public_exponent=65537,
        key_size=key_size,
    )
    public_key = private_key.public_key()

    subject = issuer = x509.Name(
        [
            x509.NameAttribute(NameOID.COUNTRY_NAME, "US"),
            x509.NameAttribute(NameOID.STATE_OR_PROVINCE_NAME, "California"),
            x509.NameAttribute(NameOID.LOCALITY_NAME, "San Francisco"),
            x509.NameAttribute(NameOID.ORGANIZATION_NAME, "ZXTX"),
            x509.NameAttribute(NameOID.COMMON_NAME, "ZXTX Signing Key"),
        ]
    )

    cert = (
        x509.CertificateBuilder()
        .subject_name(subject)
        .issuer_name(issuer)
        .public_key(public_key)
        .serial_number(x509.random_serial_number())
        .not_valid_before(datetime.datetime.now(datetime.timezone.utc))
        .not_valid_after(
            datetime.datetime.now(datetime.timezone.utc) + datetime.timedelta(days=3650)
        )
        .add_extension(x509.BasicConstraints(ca=True, path_length=None), critical=True)
        .sign(private_key, hashes.SHA256())
    )

    encryption = (
        serialization.BestAvailableEncryption(password)
        if password
        else serialization.NoEncryption()
    )

    priv_path = tmp_path / f"private_key.pem"
    pub_path = tmp_path / f"public_key.pem"
    cert_path = tmp_path / f"certificate.pem"

    priv_path.write_bytes(
        private_key.private_bytes(
            encoding=serialization.Encoding.PEM,
            format=serialization.PrivateFormat.PKCS8,
            encryption_algorithm=encryption,
        )
    )
    pub_path.write_bytes(
        public_key.public_bytes(
            encoding=serialization.Encoding.PEM,
            format=serialization.PublicFormat.SubjectPublicKeyInfo,
        )
    )
    cert_path.write_bytes(cert.public_bytes(serialization.Encoding.PEM))

    return key_size, priv_path, pub_path, cert_path, password


def test_sign_and_verify_success(keypair_and_cert):
    _, priv_path, _, cert_path, password = keypair_and_cert
    private_key = load_private_key(priv_path.read_bytes(), password=password)
    certificate = cert_path.read_bytes()

    data = b"Important test data to sign"
    signature = sign_data(private_key, data)
    assert verify_signature(certificate, data, signature)


def test_verify_fails_on_modified_data(keypair_and_cert):
    _, priv_path, _, cert_path, password = keypair_and_cert
    private_key = load_private_key(priv_path.read_bytes(), password=password)
    certificate = cert_path.read_bytes()

    data = b"Important test data to sign"
    signature = sign_data(private_key, data)
    tampered_data = data + b"tampered"

    assert not verify_signature(certificate, tampered_data, signature)


def test_verify_fails_on_wrong_signature(keypair_and_cert):
    _, _, _, cert_path, _ = keypair_and_cert
    certificate = cert_path.read_bytes()

    data = b"Important test data to sign"
    signature = b"not-a-valid-signature"

    assert not verify_signature(certificate, data, signature)



from enum import Enum

MAGIC_NUMBER = b"ZXTX"


class COMPRESSION_METHOD(Enum):
    """
    All compression methods supported by the zxtx format.

    Attributes:
        NONE (int): No compression applied.
        ZLIB (int): Use ZLIB compression algorithm.
        LZMA (int): Use LZMA compression algorithm.
        BROTLI (int): Use Brotli compression algorithm.
    """

    NONE = 0
    ZLIB = 1
    LZMA = 2
    BROTLI = 3


class CIPHER_METHOD(Enum):
    """
    All cipher methods supported by the zxtx format.

    Attributes:
        NONE (int): No encryption applied.
        AES256_GCM (int): Use AES-256 in Galois/Counter Mode (GCM).
        CHACHA20_POLY1305 (int): Use ChaCha20-Poly1305 authenticated encryption.
    """

    NONE = 0
    AES256_GCM = 1
    CHACHA20_POLY1305 = 2
    AES256_CTR_HMAC = 3
